@@ -3,6 +3,8 @@
 /* (C) 2015-2017 by Harald Welte <laforge@gnumonks.org>
  * All Rights Reserved
  *
+ * SPDX-License-Identifier: GPL-2.0+
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -465,6 +467,10 @@ DEFUN(xua_local_ip, xua_local_ip_cmd,
 	struct osmo_xua_server *xs = vty->index;
 
 	osmo_ss7_xua_server_set_local_host(xs, argv[0]);
+	if (osmo_ss7_xua_server_bind(xs) < 0) {
+		vty_out(vty, "Unable to bind xUA server to IP %s%s", argv[0], VTY_NEWLINE);
+		return CMD_WARNING;
+	}
 	return CMD_SUCCESS;
 }
 
@@ -489,11 +495,48 @@ static void write_one_xua(struct vty *vty, struct osmo_xua_server *xs)
 	vty_out(vty, " listen %s %u%s",
 		get_value_string(osmo_ss7_asp_protocol_vals, xs->cfg.proto),
 		xs->cfg.local.port, VTY_NEWLINE);
-	vty_out(vty, "  local-ip %s%s", xs->cfg.local.host, VTY_NEWLINE);
+	if (xs->cfg.local.host)
+		vty_out(vty, "  local-ip %s%s", xs->cfg.local.host, VTY_NEWLINE);
 	if (xs->cfg.accept_dyn_reg)
 		vty_out(vty, "  accept-asp-connections dynamic-permitted%s", VTY_NEWLINE);
 }
 
+static void vty_dump_xua_server(struct vty *vty, struct osmo_xua_server *xs)
+{
+	vty_out(vty, "xUA server for %s on %s:%u%s",
+		get_value_string(osmo_ss7_asp_protocol_vals, xs->cfg.proto),
+		xs->cfg.local.host ? xs->cfg.local.host : "0.0.0.0",
+		xs->cfg.local.port, VTY_NEWLINE);
+}
+
+/* List all addressbook entries */
+DEFUN(show_cs7_xua, show_cs7_xua_cmd,
+      "show cs7 "XUA_VAR_STR" [<0-65534>]",
+      SHOW_STR CS7_STR XUA_VAR_HELP_STR "Port Number")
+{
+	struct osmo_ss7_instance *inst;
+	struct osmo_xua_server *xs;
+	enum osmo_ss7_asp_protocol proto = parse_asp_proto(argv[0]);
+
+	llist_for_each_entry(inst, &osmo_ss7_instances, list) {
+		if (argc > 1) {
+			int port = atoi(argv[1]);
+			xs = osmo_ss7_xua_server_find(inst, proto, port);
+			if (!xs) {
+				vty_out(vty, "%% No matching server found%s", VTY_NEWLINE);
+				return CMD_WARNING;
+			}
+			vty_dump_xua_server(vty, xs);
+		} else {
+			llist_for_each_entry(xs, &inst->xua_servers, list) {
+				if (xs->cfg.proto == proto)
+					vty_dump_xua_server(vty, xs);
+			}
+		}
+	}
+	return CMD_SUCCESS;
+}
+
 
 /***********************************************************************
  * Application Server Process
@@ -674,7 +717,7 @@ DEFUN(cs7_as, cs7_as_cmd,
 	enum osmo_ss7_asp_protocol protocol = parse_asp_proto(argv[1]);
 
 	if (protocol == OSMO_SS7_ASP_PROT_NONE) {
-		vty_out(vty, "invalid protocol '%s'%s", argv[3], VTY_NEWLINE);
+		vty_out(vty, "invalid protocol '%s'%s", argv[1], VTY_NEWLINE);
 		return CMD_WARNING;
 	}
 
@@ -926,7 +969,7 @@ static void write_one_as(struct vty *vty, struct osmo_ss7_as *as)
 	vty_out(vty, "%s", VTY_NEWLINE);
 
 	if (as->cfg.pc_override.dpc)
-		vty_out(vty, " point-code override dpc %s%s",
+		vty_out(vty, "  point-code override dpc %s%s",
 			osmo_ss7_pointcode_print(as->inst, as->cfg.pc_override.dpc), VTY_NEWLINE);
 }
 
@@ -1246,8 +1289,9 @@ DEFUN(cs7_sccpaddr, cs7_sccpaddr_cmd,
 	const char *name = argv[0];
 
 	if (strlen(name) >= sizeof(entry->name)) {
-		vty_out(vty, "sccp address name to long!%s", VTY_NEWLINE);
-		return CMD_WARNING;
+		vty_out(vty, "Error: SCCP address name to long: '%s'%s",
+			name, VTY_NEWLINE);
+		return CMD_ERR_INCOMPLETE;
 	}
 
 	/* Ensure that we do not use address names that
@@ -1255,9 +1299,9 @@ DEFUN(cs7_sccpaddr, cs7_sccpaddr_cmd,
 	entry = addr_entry_by_name_global(name);
 	if (entry != NULL) {
 		vty_out(vty,
-			"address name (%s) already used in ss7 instance %u%s",
-			entry->name, entry->inst->cfg.id, VTY_NEWLINE);
-		return CMD_WARNING;
+			"Error: SCCP address name already used in ss7 instance %u: '%s'%s",
+			entry->inst->cfg.id, entry->name, VTY_NEWLINE);
+		return CMD_ERR_INCOMPLETE;
 	}
 
 	entry = addr_entry_by_name_local(name, inst);
@@ -1683,7 +1727,6 @@ int osmo_ss7_is_config_node(struct vty *vty, int node)
 static void vty_init_addr(void)
 {
 	install_node(&sccpaddr_node, NULL);
-	vty_install_default(L_CS7_SCCPADDR_NODE);
 	install_element(L_CS7_NODE, &cs7_show_sccpaddr_cmd);
 	install_element(L_CS7_NODE, &cs7_sccpaddr_cmd);
 	install_element(L_CS7_NODE, &cs7_sccpaddr_del_cmd);
@@ -1706,7 +1749,6 @@ static void vty_init_addr(void)
 #endif
 	install_element(L_CS7_SCCPADDR_NODE, &cs7_sccpaddr_gt_cmd);
 	install_node(&sccpaddr_gt_node, NULL);
-	vty_install_default(L_CS7_SCCPADDR_GT_NODE);
 	install_element(L_CS7_SCCPADDR_GT_NODE, &cs7_sccpaddr_gt_gti_cmd);
 	install_element(L_CS7_SCCPADDR_GT_NODE, &cs7_sccpaddr_gt_tt_cmd);
 	install_element(L_CS7_SCCPADDR_GT_NODE, &cs7_sccpaddr_gt_npi_cmd);
@@ -1719,12 +1761,12 @@ static void vty_init_shared(void *ctx)
 	g_ctx = ctx;
 
 	install_element_ve(&show_cs7_user_cmd);
+	install_element_ve(&show_cs7_xua_cmd);
 
 	/* the mother of all VTY config nodes */
 	install_element(CONFIG_NODE, &cs7_instance_cmd);
 
 	install_node(&cs7_node, config_write_cs7);
-	vty_install_default(L_CS7_NODE);
 	install_element(L_CS7_NODE, &cfg_description_cmd);
 	install_element(L_CS7_NODE, &cs7_net_ind_cmd);
 	install_element(L_CS7_NODE, &cs7_point_code_cmd);
@@ -1734,7 +1776,6 @@ static void vty_init_shared(void *ctx)
 	install_element(L_CS7_NODE, &cs7_permit_dyn_rkm_cmd);
 
 	install_node(&asp_node, NULL);
-	vty_install_default(L_CS7_ASP_NODE);
 	install_element_ve(&show_cs7_asp_cmd);
 	install_element(L_CS7_NODE, &cs7_asp_cmd);
 	install_element(L_CS7_NODE, &no_cs7_asp_cmd);
@@ -1746,7 +1787,6 @@ static void vty_init_shared(void *ctx)
 	install_element(L_CS7_ASP_NODE, &asp_shutdown_cmd);
 
 	install_node(&as_node, NULL);
-	vty_install_default(L_CS7_AS_NODE);
 	install_element_ve(&show_cs7_as_cmd);
 	install_element(L_CS7_NODE, &cs7_as_cmd);
 	install_element(L_CS7_NODE, &no_cs7_as_cmd);
@@ -1777,7 +1817,6 @@ void osmo_ss7_vty_init_sg(void *ctx)
 	vty_init_shared(ctx);
 
 	install_node(&rtable_node, NULL);
-	vty_install_default(L_CS7_RTABLE_NODE);
 	install_element_ve(&show_cs7_route_cmd);
 	install_element(L_CS7_NODE, &cs7_route_table_cmd);
 	install_element(L_CS7_RTABLE_NODE, &cfg_description_cmd);
@@ -1785,7 +1824,6 @@ void osmo_ss7_vty_init_sg(void *ctx)
 	install_element(L_CS7_RTABLE_NODE, &cs7_rt_rem_cmd);
 
 	install_node(&xua_node, NULL);
-	vty_install_default(L_CS7_XUA_NODE);
 	install_element(L_CS7_NODE, &cs7_xua_cmd);
 	install_element(L_CS7_NODE, &no_cs7_xua_cmd);
 	install_element(L_CS7_XUA_NODE, &xua_local_ip_cmd);
