@@ -6,6 +6,8 @@
  *
  * All Rights Reserved
  *
+ * SPDX-License-Identifier: GPL-2.0+
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -246,7 +248,7 @@ int _sccp_parse_connection_released(struct msgb *msgb, struct sccp_parse_result
 
 	/* we don't have enough size for the struct */
 	if (msgb_l2len(msgb) < header_size) {
-		LOGP(DSCCP, LOGL_ERROR, "msgb > header_size %u %u\n",
+		LOGP(DSCCP, LOGL_ERROR, "msgb > header_size %u %zu\n",
 		        msgb_l2len(msgb), header_size);
 		return -1;
 	}
@@ -352,7 +354,7 @@ int _sccp_parse_connection_release_complete(struct msgb *msgb, struct sccp_parse
 
 	/* header check */
 	if (msgb_l2len(msgb) < header_size) {
-		LOGP(DSCCP, LOGL_ERROR, "msgb < header_size %u %u\n",
+		LOGP(DSCCP, LOGL_ERROR, "msgb < header_size %u %zu\n",
 		        msgb_l2len(msgb), header_size);
 		return -1;
 	}
@@ -373,7 +375,7 @@ int _sccp_parse_connection_dt1(struct msgb *msgb, struct sccp_parse_result *resu
 
 	/* we don't have enough size for the struct */
 	if (msgb_l2len(msgb) < header_size) {
-		LOGP(DSCCP, LOGL_ERROR, "msgb > header_size %u %u\n",
+		LOGP(DSCCP, LOGL_ERROR, "msgb > header_size %u %zu\n",
 		        msgb_l2len(msgb), header_size);
 		return -1;
 	}
@@ -510,8 +512,10 @@ int sccp_create_sccp_addr(struct msgb *msg, const struct sockaddr_sccp *sock)
 	msgb_v_put(msg, sock->sccp_ssn);
 
 	/* copy the gti if it is present */
-	gti = msgb_put(msg, sock->gti_len);
-	memcpy(gti, sock->gti, sock->gti_len);
+	if (sock->gti) {
+		gti = msgb_put(msg, sock->gti_len);
+		memcpy(gti, sock->gti, sock->gti_len);
+	}
 
 	/* update the length now */
 	len[0] = msg->tail - len - 1;
@@ -1392,14 +1396,26 @@ osmo_static_assert(sizeof(struct sccp_source_reference) <= sizeof(uint32_t), eno
 uint32_t sccp_src_ref_to_int(struct sccp_source_reference *ref)
 {
 	uint32_t src_ref = 0;
+#if OSMO_IS_LITTLE_ENDIAN
 	memcpy(&src_ref, ref, sizeof(*ref));
+#elif OSMO_IS_BIG_ENDIAN
+	*(((uint8_t*)(&src_ref))+3) = ref->octet1;
+	*(((uint8_t*)(&src_ref))+2) = ref->octet2;
+	*(((uint8_t*)(&src_ref))+1) = ref->octet3;
+#endif
 	return src_ref;
 }
 
 struct sccp_source_reference sccp_src_ref_from_int(uint32_t int_ref)
 {
 	struct sccp_source_reference ref;
+#if OSMO_IS_LITTLE_ENDIAN
 	memcpy(&ref, &int_ref, sizeof(ref));
+#elif OSMO_IS_BIG_ENDIAN
+	ref.octet1 = *(((uint8_t*)(&int_ref))+3);
+	ref.octet2 = *(((uint8_t*)(&int_ref))+2);
+	ref.octet3 = *(((uint8_t*)(&int_ref))+1);
+#endif
 	return ref;
 }
 
