@@ -3,6 +3,8 @@
 /* (C) 2017 by Harald Welte <laforge@gnumonks.org>
  * All Rights Reserved
  *
+ * SPDX-License-Identifier: GPL-2.0+
+ *
  * based on my 2011 Erlang implementation osmo_ss7/src/sua_sccp_conv.erl
  *
  * References: ITU-T Q.713 and IETF RFC 3868
@@ -291,6 +293,27 @@ bool osmo_sccp_check_addr(struct osmo_sccp_addr *addr, uint32_t presence)
 	return true;
 }
 
+/*! Compose a human readable string to describe the SCCP user's connection.
+ * The output follows ['<scu.name>':]<local-sccp-addr>, e.g.  "'OsmoHNBW':RI=SSN_PC,PC=0.23.5,SSN=RANAP",
+ * or just "RI=SSN_PC,PC=0.23.5,SSN=RANAP" if no scu->name is set.
+ * This calls osmo_sccp_addr_name(), which returns a static buffer; hence calling this function and
+ * osmo_sccp_addr_name() in the same printf statement is likely to conflict. */
+const char *osmo_sccp_user_name(struct osmo_sccp_user *scu)
+{
+	static char buf[128];
+	struct osmo_sccp_addr sca;
+	/* Interestingly enough, the osmo_sccp_user stores an SSN and PC, but not in an osmo_sccp_addr
+	 * struct. To be able to use osmo_sccp_addr_name(), we need to first create an osmo_sccp_addr. */
+	osmo_sccp_make_addr_pc_ssn(&sca, scu->pc, scu->ssn);
+	snprintf(buf, sizeof(buf),
+		 "%s%s%s",
+		 scu->name && *scu->name ? scu->name : "",
+		 scu->name && *scu->name ? ":" : "",
+		 osmo_sccp_addr_name(scu->inst->ss7, &sca));
+	buf[sizeof(buf)-1] = '\0';
+	return buf;
+}
+
 /***********************************************************************
  * Convenience function for CLIENT
  ***********************************************************************/
@@ -519,6 +542,7 @@ osmo_sccp_simple_server_on_ss7_id(void *ctx, uint32_t ss7_id, uint32_t pc,
 {
 	struct osmo_ss7_instance *ss7;
 	struct osmo_xua_server *xs;
+	int rc;
 
 	if (local_port < 0)
 		local_port = osmo_ss7_asp_protocol_port(prot);
@@ -533,6 +557,10 @@ osmo_sccp_simple_server_on_ss7_id(void *ctx, uint32_t ss7_id, uint32_t pc,
 	if (!xs)
 		goto out_ss7;
 
+	rc = osmo_ss7_xua_server_bind(xs);
+	if (rc < 0)
+		goto out_xs;
+
 	/* Allocate SCCP stack */
 	ss7->sccp = osmo_sccp_instance_create(ss7, NULL);
 	if (!ss7->sccp)
@@ -583,7 +611,6 @@ osmo_sccp_simple_server_add_clnt(struct osmo_sccp_instance *inst,
 	as = osmo_ss7_as_find_or_create(ss7, as_name, prot);
 	if (!as)
 		goto out_strings;
-	talloc_free(as_name);
 
 	/* route only selected PC to the client */
 	rt = osmo_ss7_route_create(ss7->rtable_system, pc, 0xffff, as_name);
@@ -596,6 +623,7 @@ osmo_sccp_simple_server_add_clnt(struct osmo_sccp_instance *inst,
 	asp->cfg.is_server = true;
 	osmo_ss7_as_add_asp(as, asp_name);
 	talloc_free(asp_name);
+	talloc_free(as_name);
 	osmo_ss7_asp_restart(asp);
 
 	return ss7->sccp;
