@@ -450,6 +450,7 @@ void sccp_write_cb(struct sccp_connection *conn, struct msgb *data, void *gctx,
 	}
 
 	write_called = 1;
+	msgb_free(data);
 }
 
 void sccp_c_read(struct sccp_connection *connection, struct msgb *msgb, unsigned int len)
@@ -502,6 +503,7 @@ static void sccp_udt_write_cb(struct sccp_connection *conn, struct msgb *data, v
 	}
 
 	matched = 1;
+	msgb_free(data);
 }
 
 static void test_sccp_system(void)
@@ -884,15 +886,17 @@ static void test_sccp_parsing(void)
 				FAIL("GTI length is wrong: %d\n", result.called.gti_len);
 			}
 
-			if (memcmp(&parse_result[current_test].dst_gti_data[0],
-				   result.called.gti_data, result.called.gti_len) != 0) {
+			if (parse_result[current_test].dst_gti_data
+			    && memcmp(&parse_result[current_test].dst_gti_data[0],
+				      result.called.gti_data, result.called.gti_len) != 0) {
 				FAIL("GTI data is wrong: %d '%s'\n",
 				     result.called.gti_len,
 				     osmo_hexdump(result.called.gti_data, result.called.gti_len));
 			}
 
-			if (memcmp(&parse_result[current_test].src_gti_data[0],
-				   result.calling.gti_data, result.calling.gti_len) != 0) {
+			if (parse_result[current_test].src_gti_data
+			    && memcmp(&parse_result[current_test].src_gti_data[0],
+				      result.calling.gti_data, result.calling.gti_len) != 0) {
 				FAIL("GTI data is wrong: %d\n", result.calling.gti_len);
 			}
 		}
@@ -981,6 +985,8 @@ static void test_sccp_address(void)
 				osmo_hexdump(msg->data, ret));
 		}
 	}
+
+	talloc_free(msg);
 }
 
 static const struct log_info_cat default_categories[] = {
